<?php
declare(strict_types=1);

namespace Catequil\Db;

use Catequil\Cache\Service\CacheManager;
use Doctrine\Common\EventManager;
use Symfony\Component\Cache\Adapter\RedisAdapter;

/**
 * Class ConfigProvider
 *
 * @package Catequil\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'db' => $this->getDbConfig(),
            'dependencies' => $this->getDependencies(),
            CacheManager::class => [
                'db.cache' => [
                    'adapter' => RedisAdapter::class,
                    'options' => [
                        'namespace' => __DIR__,
                        'database' => 1,
                    ],
                ],
            ],
        ];
    }

    public function getDbConfig(): array
    {
        return [
            'default_connection' => 'db.default',
            'connections' => [
                'db.default' => [
                    'driver' => 'pdo_mysql',
                    'host' => 'localhost',
                    'user' => '',
                    'password' => '',
                    'charset' => 'utf8mb4',
                ],
            ],
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                \Doctrine\DBAL\Connection::class => Connection::class,
                Connection::class => 'db',
            ],
            'invokables' => [
                EventManager::class,
            ],
            'abstract_factories' => [
                Container\ConnectionAbstractFactory::class,
            ],
        ];
    }
}
