<?php
namespace Catequil\Db\Container;

use Doctrine\DBAL\Configuration;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerInterface;
use Symfony\Component\Cache\Adapter\ArrayAdapter;

/**
 * Class ConfigurationFactory
 *
 * @package Catequil\Db\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConfigurationFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return Configuration
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): Configuration
    {
        $config = $container->get('config');
        $isDevelopment = $config['development'] ?? false;

        $configuration = new Configuration();
        $cache = $isDevelopment
            ? new ArrayAdapter()
            : $container->get('cache.db');

        $configuration->setResultCache($cache);

        return $configuration;
    }
}