<?php
declare(strict_types=1);

namespace CatequilTest\Db;

use Ak\Test\AbstractTestCase;
use Catequil\Db\ConfigProvider;
use Catequil\Db\Connection;
use Doctrine\DBAL\Driver;
use Laminas\ConfigAggregator\ArrayProvider;
use PHPUnit\Framework\MockObject\MockObject;
use ReflectionClass;
use ReflectionException;

/**
 * Class AbstractDbTestCase
 *
 * @package CatequilTest\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
abstract class AbstractDbTestCase extends AbstractTestCase
{
    protected array $providers = [
        ConfigProvider::class,
    ];

    private ?Driver\Connection $connection = null;
    private ?Driver\Statement $statement = null;
    private ?Driver\Result $result = null;

    public function __construct(?string $name = null, array $data = [], $dataName = '')
    {
        parent::__construct($name, $data, $dataName);

        $this->providers[] = new ArrayProvider([
            'db' => [
                'connections' => [
                    'db.default' => [
                        'host' => '127.0.0.1',
                        'user' => 'root',
                        'password' => 'root',
                        'dbname' => 'test',
                    ],
                    'db.memory' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);
    }

    /**
     * @param string $name
     *
     * @return Connection
     * @noinspection PhpDocMissingThrowsInspection
     */
    protected function getConnection(string $name = 'db.default'): Connection
    {
        $this->connection = null;
        $this->statement = null;
        $this->result = null;

        $container = $this->getContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $conn = $container->get($name);

        try {
            $class = new ReflectionClass($conn);
            $property = $class->getProperty('_conn');
            $property->setValue($conn, $this->connection());
        } catch (ReflectionException) {
        }

        return $conn;
    }

    /**
     * @return MockObject
     */
    protected function connection(): MockObject
    {
        if ($this->connection === null) {
            $this->connection = $this->createMock(Driver\Connection::class);
        }
        return $this->connection;
    }

    protected function statement(): MockObject
    {
        if ($this->statement === null) {
            $this->statement = $this->createMock(Driver\Statement::class);
        }
        return $this->statement;
    }

    protected function result(): MockObject
    {
        if ($this->result === null) {
            $this->result = $this->createMock(Driver\Result::class);
        }
        return $this->result;
    }
}
